@@ -1,25 +1,48 @@
-from __future__ import unicode_literals
-
-import re
 import functools
+import re
+from typing import (
+    Any,
+    Callable,
+    Union,
+)
+import warnings
 
+from eth_typing import (
+    Address,
+    ChecksumAddress,
+)
 from eth_utils import (
     is_string,
-    coerce_args_to_text,
-    pad_left,
-    add_0x_prefix,
+    to_checksum_address,
 )
 
-from web3.utils.validation import (
+from web3._utils.compat import (
+    TypedDict,
+)
+from web3._utils.validation import (
     validate_address,
 )
 
-
-def pad_left_hex(value, num_bytes):
-    return pad_left(value, num_bytes * 2, '0')
+IbanOptions = TypedDict("IbanOptions", {
+    "institution": str,
+    "identifier": str,
+})
 
 
-def iso13616Prepare(iban):
+def _issue_deprecation_warning() -> None:
+    warnings.warn(
+        "The IBAN feature is deprecated and will be removed in v6",
+        category=DeprecationWarning,
+    )
+
+
+def pad_left_hex(value: str, num_bytes: int) -> str:
+    _issue_deprecation_warning()
+
+    return value.rjust(num_bytes * 2, "0")
+
+
+def iso13616Prepare(iban: str) -> str:
     """
     Prepare an IBAN for mod 97 computation by moving the first
     4 chars to the end and transforming the letters to numbers
@@ -29,13 +52,15 @@ def iso13616Prepare(iban):
     @param {String} iban the IBAN
     @returns {String} the prepared IBAN
     """
+    _issue_deprecation_warning()
+
     A = ord("A")
     Z = ord("Z")
 
     iban = iban.upper()
     iban = iban[4:] + iban[:4]
 
-    def charfunc(n):
+    def charfunc(n: str) -> str:
         code = ord(n)
         if code >= A and code <= Z:
             return str(code - A + 10)
@@ -45,7 +70,7 @@ def iso13616Prepare(iban):
     return "".join(map(charfunc, list(iban)))
 
 
-def mod9710(iban):
+def mod9710(iban: str) -> int:
     """
     Calculates the MOD 97 10 of the passed IBAN as specified in ISO7064.
 
@@ -53,6 +78,8 @@ def mod9710(iban):
     @param {String} iban
     @returns {Number}
     """
+    _issue_deprecation_warning()
+
     remainder = iban
     block = None
 
@@ -63,18 +90,20 @@ def mod9710(iban):
     return int(remainder) % 97
 
 
-def baseN(num, b, numerals="0123456789abcdefghijklmnopqrstuvwxyz"):
+def baseN(num: int, b: int, numerals: str = "0123456789abcdefghijklmnopqrstuvwxyz") -> str:
     """
     This prototype should be used to create
     an iban object from iban correct string
 
     @param {String} iban
     """
+    _issue_deprecation_warning()
+
     return ((num == 0) and numerals[0]) or \
         (baseN(num // b, b, numerals).lstrip(numerals[0]) + numerals[num % b])
 
 
-class IsValid(object):
+class IsValid:
     """
     Should be called to check if iban is correct
 
@@ -84,14 +113,17 @@ class IsValid(object):
     @method isValid
     @returns {Boolean} true if it is, otherwise false
     """
-    def __get__(self, instance, owner):
+    def __get__(self, instance: 'Iban', owner: str) -> Callable[[str], bool]:
+        _issue_deprecation_warning()
+
         if instance is None:
             return self.validate
         return functools.partial(self.validate, instance._iban)
 
     @staticmethod
-    @coerce_args_to_text
-    def validate(iban_address):
+    def validate(iban_address: Any) -> bool:
+        _issue_deprecation_warning()
+
         if not is_string(iban_address):
             return False
 
@@ -102,12 +134,14 @@ class IsValid(object):
         return False
 
 
-class Iban(object):
-    def __init__(self, iban):
+class Iban:
+    def __init__(self, iban: str) -> None:
+        _issue_deprecation_warning()
+
         self._iban = iban
 
     @staticmethod
-    def fromAddress(address):
+    def fromAddress(address: Union[Address, ChecksumAddress]) -> "Iban":
         """
         This method should be used to create
         an iban object from ethereum address
@@ -116,6 +150,8 @@ class Iban(object):
         @param {String} address
         @return {Iban} the IBAN object
         """
+        _issue_deprecation_warning()
+
         validate_address(address)
         address_as_integer = int(address, 16)
         address_as_base36 = baseN(address_as_integer, 36)
@@ -123,7 +159,7 @@ class Iban(object):
         return Iban.fromBban(padded.upper())
 
     @staticmethod
-    def fromBban(bban):
+    def fromBban(bban: str) -> "Iban":
         """
         Convert the passed BBAN to an IBAN for this country specification.
         Please note that <i>"generation of the IBAN shall be the exclusive
@@ -135,6 +171,8 @@ class Iban(object):
         @param {String} bban the BBAN to convert to IBAN
         @returns {Iban} the IBAN object
         """
+        _issue_deprecation_warning()
+
         countryCode = "XE"
 
         remainder = mod9710(iso13616Prepare(countryCode + "00" + bban))
@@ -143,7 +181,7 @@ class Iban(object):
         return Iban(countryCode + checkDigit + bban)
 
     @staticmethod
-    def createIndirect(options):
+    def createIndirect(options: IbanOptions) -> "Iban":
         """
         Should be used to create IBAN object for given institution and identifier
 
@@ -151,29 +189,35 @@ class Iban(object):
         @param {Object} options, required options are "institution" and "identifier"
         @return {Iban} the IBAN object
         """
+        _issue_deprecation_warning()
+
         return Iban.fromBban("ETH" + options["institution"] + options["identifier"])
 
     isValid = IsValid()
 
-    def isDirect(self):
+    def isDirect(self) -> bool:
         """
         Should be called to check if iban number is direct
 
         @method isDirect
         @returns {Boolean} true if it is, otherwise false
         """
+        _issue_deprecation_warning()
+
         return len(self._iban) in [34, 35]
 
-    def isIndirect(self):
+    def isIndirect(self) -> bool:
         """
         Should be called to check if iban number if indirect
 
         @method isIndirect
         @returns {Boolean} true if it is, otherwise false
         """
+        _issue_deprecation_warning()
+
         return len(self._iban) == 20
 
-    def checksum(self):
+    def checksum(self) -> str:
         """
         Should be called to get iban checksum
         Uses the mod-97-10 checksumming protocol (ISO/IEC 7064:2003)
@@ -181,9 +225,11 @@ class Iban(object):
         @method checksum
         @returns {String} checksum
         """
+        _issue_deprecation_warning()
+
         return self._iban[2:4]
 
-    def institution(self):
+    def institution(self) -> str:
         """
         Should be called to get institution identifier
         eg. XREG
@@ -191,12 +237,14 @@ class Iban(object):
         @method institution
         @returns {String} institution identifier
         """
+        _issue_deprecation_warning()
+
         if self.isIndirect():
             return self._iban[7:11]
         else:
             return ""
 
-    def client(self):
+    def client(self) -> str:
         """
         Should be called to get client identifier within institution
         eg. GAVOFYORK
@@ -204,24 +252,30 @@ class Iban(object):
         @method client
         @returns {String} client identifier
         """
+        _issue_deprecation_warning()
+
         if self.isIndirect():
             return self._iban[11:]
         else:
             return ""
 
-    def address(self):
+    def address(self) -> Union[str, ChecksumAddress]:
         """
         Should be called to get client direct address
 
         @method address
         @returns {String} client direct address
         """
+        _issue_deprecation_warning()
+
         if self.isDirect():
             base36 = self._iban[4:]
             asInt = int(base36, 36)
-            return add_0x_prefix(pad_left_hex(baseN(asInt, 16), 20))
+            return to_checksum_address(pad_left_hex(baseN(asInt, 16), 20))
 
         return ""
 
-    def toString(self):
+    def toString(self) -> str:
+        _issue_deprecation_warning()
+
         return self._iban
